@@ -21,6 +21,11 @@ BACKUP_VTHUNDER = 'backup_vthunder'
 VRRP_STATUS = 'vrrp_status'
 MASTER_VRRP_STATUS = 'master_vrrp_status'
 BACKUP_VRRP_STATUS = 'backup_vrrp_status'
+NAT_POOL = 'nat_pool'
+NAT_FLAVOR = 'nat_flavor'
+SUBNET_PORT = 'subnet_port'
+WRITE_MEM_SHARED = 'write_mem_shared'
+WRITE_MEM_PRIVATE = 'write_mem_private'
 
 FAILED = 'FAILED'
 USED_SPARE = 'USED_SPARE'
@@ -105,7 +110,10 @@ DELETE_HEALTH_MONITOR_SUBFLOW_WITH_POOL_DELETE_FLOW = 'delete-health-monitor-sub
 DELETE_MEMBERS_SUBFLOW_WITH_POOL_DELETE_FLOW = 'delete-members-subflow-with-pool-delete-flow'
 HANDLE_VRID_MEMBER_SUBFLOW = 'handle-vrid-member-subflow'
 SPARE_VTHUNDER_CREATE = 'spare-vthunder-create'
+WRITE_MEMORY_THUNDER_FLOW = 'write-memory-thunder-flow'
+RELOAD_CHECK_THUNDER_FLOW = 'reload-check-thunder-flow'
 LB_TO_VTHUNDER_SUBFLOW = 'lb-to-vthunder-subflow'
+LOADBALANCERS_LIST = 'loadbalancers_list'
 VRID_LIST = 'vrid_list'
 RESOURCE_COUNT = 'resource_count'
 
