@@ -33,6 +33,7 @@ from a10_octavia.controller.worker.flows import vthunder_flows
 from a10_octavia.controller.worker.tasks import a10_compute_tasks
 from a10_octavia.controller.worker.tasks import a10_database_tasks
 from a10_octavia.controller.worker.tasks import a10_network_tasks
+from a10_octavia.controller.worker.tasks import nat_pool_tasks
 from a10_octavia.controller.worker.tasks import virtual_server_tasks
 from a10_octavia.controller.worker.tasks import vthunder_tasks
 
@@ -79,6 +80,8 @@ class LoadBalancerFlows(object):
                            a10constants.VTHUNDER)))
         lb_create_flow.add(vthunder_tasks.WriteMemory(
             requires=a10constants.VTHUNDER))
+        lb_create_flow.add(a10_database_tasks.SetThunderUpdatedAt(
+            requires=a10constants.VTHUNDER))
         return lb_create_flow
 
     def _create_single_topology(self):
@@ -163,12 +166,23 @@ class LoadBalancerFlows(object):
             requires=constants.SERVER_GROUP_ID))
         delete_LB_flow.add(database_tasks.MarkLBAmphoraeHealthBusy(
             requires=constants.LOADBALANCER))
+        delete_LB_flow.add(a10_database_tasks.GetFlavorData(
+            rebind={a10constants.LB_RESOURCE: constants.LOADBALANCER},
+            provides=constants.FLAVOR_DATA))
+        delete_LB_flow.add(a10_database_tasks.CountLoadbalancersWithFlavor(
+            requires=constants.LOADBALANCER, provides=a10constants.LB_COUNT))
+        delete_LB_flow.add(nat_pool_tasks.NatPoolDelete(
+            requires=(constants.LOADBALANCER,
+                      a10constants.VTHUNDER, a10constants.LB_COUNT, constants.FLAVOR_DATA)))
         delete_LB_flow.add(virtual_server_tasks.DeleteVirtualServerTask(
             requires=(constants.LOADBALANCER, a10constants.VTHUNDER)))
+        delete_LB_flow.add(self.get_delete_lb_vrid_subflow())
         if CONF.a10_global.network_type == 'vlan':
-            delete_LB_flow.add(vthunder_tasks.DeleteInterfaceTagIfNotInUseForLB(
-                requires=[constants.LOADBALANCER,
-                          a10constants.VTHUNDER]))
+            delete_LB_flow.add(
+                vthunder_tasks.DeleteInterfaceTagIfNotInUseForLB(
+                    requires=[
+                        constants.LOADBALANCER,
+                        a10constants.VTHUNDER]))
 
         # delete_LB_flow.add(listeners_delete)
         # delete_LB_flow.add(network_tasks.UnplugVIP(
@@ -192,6 +206,8 @@ class LoadBalancerFlows(object):
             requires=constants.LOADBALANCER))
         delete_LB_flow.add(vthunder_tasks.WriteMemory(
             requires=a10constants.VTHUNDER))
+        delete_LB_flow.add(a10_database_tasks.SetThunderUpdatedAt(
+            requires=a10constants.VTHUNDER))
         return (delete_LB_flow, store)
 
     def get_new_lb_networking_subflow(self, topology):
@@ -228,17 +244,21 @@ class LoadBalancerFlows(object):
             requires=a10constants.VTHUNDER,
             inject={a10constants.STATUS: constants.ACTIVE}))
         if topology == constants.TOPOLOGY_ACTIVE_STANDBY:
-            new_LB_net_subflow.add(vthunder_tasks.CreateHealthMonitorOnVThunder(
-                name=a10constants.CREATE_HEALTH_MONITOR_ON_VTHUNDER_MASTER,
-                requires=a10constants.VTHUNDER))
-            new_LB_net_subflow.add(a10_database_tasks.GetBackupVThunderByLoadBalancer(
-                name=a10constants.GET_BACKUP_VTHUNDER_BY_LB,
-                requires=constants.LOADBALANCER,
-                provides=a10constants.BACKUP_VTHUNDER))
-            new_LB_net_subflow.add(vthunder_tasks.VThunderComputeConnectivityWait(
-                name=a10constants.BACKUP_CONNECTIVITY_WAIT,
-                rebind={a10constants.VTHUNDER: a10constants.BACKUP_VTHUNDER},
-                requires=constants.AMPHORA))
+            new_LB_net_subflow.add(
+                vthunder_tasks.CreateHealthMonitorOnVThunder(
+                    name=a10constants.CREATE_HEALTH_MONITOR_ON_VTHUNDER_MASTER,
+                    requires=a10constants.VTHUNDER))
+            new_LB_net_subflow.add(
+                a10_database_tasks.GetBackupVThunderByLoadBalancer(
+                    name=a10constants.GET_BACKUP_VTHUNDER_BY_LB,
+                    requires=constants.LOADBALANCER,
+                    provides=a10constants.BACKUP_VTHUNDER))
+            new_LB_net_subflow.add(
+                vthunder_tasks.VThunderComputeConnectivityWait(
+                    name=a10constants.BACKUP_CONNECTIVITY_WAIT,
+                    rebind={
+                        a10constants.VTHUNDER: a10constants.BACKUP_VTHUNDER},
+                    requires=constants.AMPHORA))
             new_LB_net_subflow.add(vthunder_tasks.EnableInterface(
                 name=a10constants.BACKUP_ENABLE_INTERFACE,
                 rebind={a10constants.VTHUNDER: a10constants.BACKUP_VTHUNDER}))
@@ -267,8 +287,14 @@ class LoadBalancerFlows(object):
             requires=(constants.LOADBALANCER, constants.UPDATE_DICT)))
         # update_LB_flow.add(amphora_driver_tasks.ListenersUpdate(
         #    requires=[constants.LOADBALANCER, constants.LISTENERS]))
+        # post_create_lb_flow.add(handle_vrid_for_loadbalancer_subflow())
+        update_LB_flow.add(self.handle_vrid_for_loadbalancer_subflow())
+        update_LB_flow.add(a10_database_tasks.GetFlavorData(
+            rebind={a10constants.LB_RESOURCE: constants.LOADBALANCER},
+            provides=constants.FLAVOR_DATA))
         update_LB_flow.add(virtual_server_tasks.UpdateVirtualServerTask(
-            requires=(constants.LOADBALANCER, a10constants.VTHUNDER)))
+            requires=(constants.LOADBALANCER, a10constants.VTHUNDER,
+                      constants.FLAVOR_DATA)))
         update_LB_flow.add(database_tasks.UpdateLoadbalancerInDB(
             requires=[constants.LOADBALANCER, constants.UPDATE_DICT]))
         if CONF.a10_global.network_type == 'vlan':
@@ -279,9 +305,16 @@ class LoadBalancerFlows(object):
             requires=constants.LOADBALANCER))
         update_LB_flow.add(vthunder_tasks.WriteMemory(
             requires=a10constants.VTHUNDER))
+        update_LB_flow.add(a10_database_tasks.MarkVThunderStatusInDB(
+            name="pending_update_to_active",
+            requires=a10constants.VTHUNDER,
+            inject={"status": constants.ACTIVE}))
+        update_LB_flow.add(a10_database_tasks.SetThunderUpdatedAt(
+            requires=a10constants.VTHUNDER))
         return update_LB_flow
 
-    def get_create_rack_vthunder_load_balancer_flow(self, vthunder_conf, topology, listeners=None):
+    def get_create_rack_vthunder_load_balancer_flow(
+            self, vthunder_conf, topology, listeners=None):
         """Flow to create rack load balancer"""
 
         f_name = constants.CREATE_LOADBALANCER_FLOW
@@ -289,22 +322,45 @@ class LoadBalancerFlows(object):
 
         lb_create_flow.add(lifecycle_tasks.LoadBalancerIDToErrorOnRevertTask(
             requires=constants.LOADBALANCER_ID))
-
-        lb_create_flow.add(self.vthunder_flows.get_rack_vthunder_for_lb_subflow(
-            vthunder_conf=vthunder_conf,
-            prefix=constants.ROLE_STANDALONE,
-            role=constants.ROLE_STANDALONE))
+        lb_create_flow.add(database_tasks.ReloadLoadBalancer(
+            requires=constants.LOADBALANCER_ID,
+            provides=constants.LOADBALANCER))
+        lb_create_flow.add(
+            a10_database_tasks.CheckExistingProjectToThunderMappedEntries(
+                inject={
+                    a10constants.VTHUNDER_CONFIG: vthunder_conf},
+                requires=(
+                    constants.LOADBALANCER,
+                    a10constants.VTHUNDER_CONFIG),
+                provides=a10constants.VTHUNDER_CONFIG))
+        lb_create_flow.add(
+            a10_database_tasks.CheckExistingThunderToProjectMappedEntries(
+                requires=(
+                    constants.LOADBALANCER,
+                    a10constants.VTHUNDER_CONFIG)))
+        lb_create_flow.add(
+            self.vthunder_flows.get_rack_vthunder_for_lb_subflow(
+                vthunder_conf=a10constants.VTHUNDER_CONFIG,
+                prefix=constants.ROLE_STANDALONE,
+                role=constants.ROLE_STANDALONE))
         post_amp_prefix = constants.POST_LB_AMP_ASSOCIATION_SUBFLOW
         lb_create_flow.add(
             self.get_post_lb_rack_vthunder_association_flow(
                 post_amp_prefix, topology, mark_active=(not listeners)))
-
+        lb_create_flow.add(a10_database_tasks.GetFlavorData(
+            rebind={a10constants.LB_RESOURCE: constants.LOADBALANCER},
+            provides=constants.FLAVOR_DATA))
+        lb_create_flow.add(nat_pool_tasks.NatPoolCreate(
+            requires=(constants.LOADBALANCER,
+                      a10constants.VTHUNDER, constants.FLAVOR_DATA)))
         lb_create_flow.add(virtual_server_tasks.CreateVirtualServerTask(
-            requires=(constants.LOADBALANCER, a10constants.VTHUNDER),
+            requires=(constants.LOADBALANCER, a10constants.VTHUNDER,
+                      constants.FLAVOR_DATA),
             provides=a10constants.STATUS))
-
         lb_create_flow.add(vthunder_tasks.WriteMemory(
             requires=a10constants.VTHUNDER))
+        lb_create_flow.add(a10_database_tasks.SetThunderUpdatedAt(
+            requires=a10constants.VTHUNDER))
         return lb_create_flow
 
     def get_post_lb_rack_vthunder_association_flow(self, prefix, topology,
@@ -321,9 +377,9 @@ class LoadBalancerFlows(object):
                 name=sf_name + '-' + constants.RELOAD_LB_AFTER_AMP_ASSOC,
                 requires=constants.LOADBALANCER_ID,
                 provides=constants.LOADBALANCER))
-
         post_create_lb_flow.add(database_tasks.UpdateLoadbalancerInDB(
             requires=[constants.LOADBALANCER, constants.UPDATE_DICT]))
+        post_create_lb_flow.add(self.handle_vrid_for_loadbalancer_subflow())
         if CONF.a10_global.network_type == 'vlan':
             post_create_lb_flow.add(vthunder_tasks.TagInterfaceForLB(
                 requires=[constants.LOADBALANCER,
@@ -333,3 +389,73 @@ class LoadBalancerFlows(object):
                 name=sf_name + '-' + constants.MARK_LB_ACTIVE_INDB,
                 requires=constants.LOADBALANCER))
         return post_create_lb_flow
+
+    def handle_vrid_for_loadbalancer_subflow(self):
+        handle_vrid_for_lb_subflow = linear_flow.Flow(
+            a10constants.HANDLE_VRID_LOADBALANCER_SUBFLOW)
+        handle_vrid_for_lb_subflow.add(a10_network_tasks.GetLBResourceSubnet(
+            rebind={a10constants.LB_RESOURCE: constants.LOADBALANCER},
+            provides=constants.SUBNET))
+        handle_vrid_for_lb_subflow.add(
+            a10_database_tasks.GetChildProjectsOfParentPartition(
+                rebind={a10constants.LB_RESOURCE: constants.LOADBALANCER},
+                provides=a10constants.PARTITION_PROJECT_LIST
+            ))
+        handle_vrid_for_lb_subflow.add(
+            a10_database_tasks.GetVRIDForLoadbalancerResource(
+                requires=a10constants.PARTITION_PROJECT_LIST,
+                provides=a10constants.VRID_LIST))
+        handle_vrid_for_lb_subflow.add(
+            a10_network_tasks.HandleVRIDFloatingIP(
+                requires=[
+                    a10constants.VTHUNDER,
+                    a10constants.VRID_LIST,
+                    constants.SUBNET],
+                rebind={
+                    a10constants.LB_RESOURCE: constants.LOADBALANCER},
+                provides=a10constants.VRID_LIST))
+        handle_vrid_for_lb_subflow.add(
+            a10_database_tasks.UpdateVRIDForLoadbalancerResource(
+                requires=a10constants.VRID_LIST, rebind={
+                    a10constants.LB_RESOURCE: constants.LOADBALANCER}))
+        return handle_vrid_for_lb_subflow
+
+    def get_delete_lb_vrid_subflow(self):
+        delete_lb_vrid_subflow = linear_flow.Flow(
+            a10constants.DELETE_LOADBALANCER_VRID_SUBFLOW)
+        delete_lb_vrid_subflow.add(a10_network_tasks.GetLBResourceSubnet(
+            rebind={a10constants.LB_RESOURCE: constants.LOADBALANCER},
+            provides=constants.SUBNET))
+        delete_lb_vrid_subflow.add(
+            a10_database_tasks.GetChildProjectsOfParentPartition(
+                requires=[a10constants.VTHUNDER],
+                rebind={a10constants.LB_RESOURCE: constants.LOADBALANCER},
+                provides=a10constants.PARTITION_PROJECT_LIST
+            ))
+        delete_lb_vrid_subflow.add(
+            a10_database_tasks.CountLoadbalancersInProjectBySubnet(
+                requires=[constants.SUBNET, a10constants.PARTITION_PROJECT_LIST],
+                provides=a10constants.LB_COUNT))
+        delete_lb_vrid_subflow.add(
+            a10_database_tasks.CountMembersInProjectBySubnet(
+                requires=[constants.SUBNET, a10constants.PARTITION_PROJECT_LIST],
+                provides=a10constants.MEMBER_COUNT))
+        delete_lb_vrid_subflow.add(
+            a10_database_tasks.GetVRIDForLoadbalancerResource(
+                requires=a10constants.PARTITION_PROJECT_LIST,
+                provides=a10constants.VRID_LIST))
+        delete_lb_vrid_subflow.add(
+            a10_network_tasks.DeleteVRIDPort(
+                requires=[
+                    a10constants.VTHUNDER,
+                    a10constants.VRID_LIST,
+                    constants.SUBNET,
+                    a10constants.LB_COUNT,
+                    a10constants.MEMBER_COUNT],
+                provides=(
+                    a10constants.VRID,
+                    a10constants.DELETE_VRID)))
+        delete_lb_vrid_subflow.add(a10_database_tasks.DeleteVRIDEntry(
+            requires=[a10constants.VRID, a10constants.DELETE_VRID]))
+
+        return delete_lb_vrid_subflow
