@@ -32,7 +32,7 @@ class CreateAndAssociateHealthMonitor(task.Task):
     """Task to create a healthmonitor and associate it with provided pool."""
 
     @axapi_client_decorator
-    def execute(self, listeners, health_mon, vthunder):
+    def execute(self, listeners, health_mon, vthunder, flavor=None):
         method = None
         url = None
         expect_code = None
@@ -42,15 +42,28 @@ class CreateAndAssociateHealthMonitor(task.Task):
             url = health_mon.url_path
             expect_code = health_mon.expected_codes
         args = utils.meta(health_mon, 'hm', {})
+        args = utils.dash_to_underscore(args)
+
+        # overwrite options from flavor
+        if flavor:
+            flavors = flavor.get('health_monitor')
+            if flavors:
+                name_exprs = flavors.get('name_expressions')
+                parsed_exprs = utils.parse_name_expressions(health_mon.name, name_exprs)
+                flavors.pop('name_expressions', None)
+                flavors.update(parsed_exprs)
+                args.update({'monitor': flavors})
 
         try:
+            post_data = CONF.health_monitor.post_data
             self.axapi_client.slb.hm.create(health_mon.id,
                                             openstack_mappings.hm_type(self.axapi_client,
                                                                        health_mon.type),
                                             health_mon.delay, health_mon.timeout,
                                             health_mon.rise_threshold, method=method,
                                             port=listeners[0].protocol_port, url=url,
-                                            expect_code=expect_code, axapi_args=args)
+                                            expect_code=expect_code, post_data=post_data,
+                                            **args)
             LOG.debug("Successfully created health monitor: %s", health_mon.id)
 
         except (acos_errors.ACOSException, ConnectionError) as e:
@@ -59,7 +72,7 @@ class CreateAndAssociateHealthMonitor(task.Task):
 
         try:
             self.axapi_client.slb.service_group.update(health_mon.pool_id,
-                                                       health_monitor=health_mon.id,
+                                                       hm_name=health_mon.id,
                                                        health_check_disable=0)
             LOG.debug("Successfully associated health monitor %s to pool %s",
                       health_mon.id, health_mon.pool_id)
@@ -88,8 +101,7 @@ class DeleteHealthMonitor(task.Task):
     def execute(self, health_mon, vthunder):
         try:
             self.axapi_client.slb.service_group.update(health_mon.pool_id,
-                                                       health_monitor="",
-                                                       health_check_disable=True)
+                                                       hm_delete=True)
             LOG.debug("Successfully dissociated health monitor %s from pool %s",
                       health_mon.id, health_mon.pool_id)
         except (acos_errors.ACOSException, ConnectionError) as e:
@@ -110,7 +122,7 @@ class UpdateHealthMonitor(task.Task):
     """Task to update Health Monitor"""
 
     @axapi_client_decorator
-    def execute(self, listeners, health_mon, vthunder, update_dict):
+    def execute(self, listeners, health_mon, vthunder, update_dict, flavor=None):
         """ Execute update health monitor """
         # TODO(hthompson6) Length of name of healthmonitor for older vThunder devices
         health_mon.update(update_dict)
@@ -122,13 +134,26 @@ class UpdateHealthMonitor(task.Task):
             url = health_mon.url_path
             expect_code = health_mon.expected_codes
         args = utils.meta(health_mon, 'hm', {})
+        args = utils.dash_to_underscore(args)
+
+        # overwrite options from flavor
+        if flavor:
+            flavors = flavor.get('health_monitor')
+            if flavors:
+                name_exprs = flavors.get('name_expressions')
+                parsed_exprs = utils.parse_name_expressions(health_mon.name, name_exprs)
+                flavors.pop('name_expressions', None)
+                flavors.update(parsed_exprs)
+                args.update({'monitor': flavors})
+
         try:
+            post_data = CONF.health_monitor.post_data
             self.axapi_client.slb.hm.update(
                 health_mon.id,
                 openstack_mappings.hm_type(self.axapi_client, health_mon.type),
                 health_mon.delay, health_mon.timeout, health_mon.rise_threshold,
-                method=method, url=url, expect_code=expect_code,
-                port=listeners[0].protocol_port, axapi_args=args)
+                method=method, url=url, expect_code=expect_code, post_data=post_data,
+                port=listeners[0].protocol_port, **args)
             LOG.debug("Successfully updated health monitor: %s", health_mon.id)
         except (acos_errors.ACOSException, ConnectionError) as e:
             LOG.exception("Failed to update health monitor: %s", health_mon.id)
