@@ -48,6 +48,7 @@ class VThunder(base_models.BASE):
     updated_at = sa.Column(u'updated_at', sa.DateTime(), nullable=True)
     partition_name = sa.Column(sa.String(14), nullable=True)
     hierarchical_multitenancy = sa.Column(sa.String(7), nullable=False)
+    last_write_mem = sa.Column(u'last_write_mem', sa.DateTime(), nullable=True)
 
     @classmethod
     def find_by_loadbalancer_id(cls, loadbalancer_id, db_session=None):
@@ -64,3 +65,19 @@ class VRID(base_models.BASE):
     vrid_port_id = sa.Column(sa.String(36), nullable=False)
     vrid_floating_ip = sa.Column(sa.String(40))
     subnet_id = sa.Column(sa.String(36), nullable=False)
+
+
+class NATPool(base_models.BASE):
+    __data_model__ = data_models.NATPool
+    __tablename__ = 'nat_pool'
+    __table_args__ = (
+        sa.UniqueConstraint('name', 'subnet_id', name='unique_name_subnet_id'),
+    )
+    id = sa.Column(sa.String(64), primary_key=True)
+    name = sa.Column(sa.String(64), nullable=False)
+    subnet_id = sa.Column(sa.String(64), nullable=False)
+    start_address = sa.Column('start_address', sa.String(64), nullable=False)
+    end_address = sa.Column('end_address', sa.String(64), nullable=False)
+    member_ref_count = sa.Column(sa.Integer, default=0, nullable=False)
+    port_id = sa.Column(sa.String(64), nullable=False)
+
