@@ -183,13 +183,21 @@ class BaseRepository(object):
 class VThunderRepository(BaseRepository):
     model_class = models.VThunder
 
+    def get_recently_updated_thunders(self, session):
+        query = session.query(self.model_class).filter(
+            or_(self.model_class.updated_at >= self.model_class.last_write_mem,
+                self.model_class.last_write_mem == None)).filter(
+            or_(self.model_class.status == 'ACTIVE', self.model_class.status == 'DELETED'))
+        query = query.options(noload('*'))
+        return query.all()
+
     def get_stale_vthunders(self, session, initial_setup_wait_time, failover_wait_time):
         model = session.query(self.model_class).filter(
             self.model_class.created_at < initial_setup_wait_time).filter(
             self.model_class.last_udp_update < failover_wait_time).filter(
-                self.model_class.status == 'ACTIVE').filter(
-                or_(self.model_class.role == "MASTER",
-                    self.model_class.role == "BACKUP")).first()
+            self.model_class.status == 'ACTIVE').filter(
+            or_(self.model_class.role == "MASTER",
+                self.model_class.role == "BACKUP")).first()
         if model is None:
             return None
         return model.to_data_model()
@@ -197,8 +205,8 @@ class VThunderRepository(BaseRepository):
     def get_vthunder_from_lb(self, session, lb_id):
         model = session.query(self.model_class).filter(
             self.model_class.loadbalancer_id == lb_id).filter(
-                or_(self.model_class.role == "STANDALONE",
-                    self.model_class.role == "MASTER")).first()
+            or_(self.model_class.role == "STANDALONE",
+                self.model_class.role == "MASTER")).first()
 
         if not model:
             return None
@@ -208,8 +216,8 @@ class VThunderRepository(BaseRepository):
     def get_backup_vthunder_from_lb(self, session, lb_id):
         model = session.query(self.model_class).filter(
             self.model_class.loadbalancer_id == lb_id).filter(
-                or_(self.model_class.role == "STANDALONE",
-                    self.model_class.role == "BACKUP")).first()
+            or_(self.model_class.role == "STANDALONE",
+                self.model_class.role == "BACKUP")).first()
 
         if not model:
             return None
@@ -219,15 +227,34 @@ class VThunderRepository(BaseRepository):
     def get_vthunder_by_project_id(self, session, project_id):
         model = session.query(self.model_class).filter(
             self.model_class.project_id == project_id).filter(
-                and_(self.model_class.status == "ACTIVE",
-                     or_(self.model_class.role == "STANDALONE",
-                         self.model_class.role == "MASTER"))).first()
+            and_(self.model_class.status == "ACTIVE",
+                 or_(self.model_class.role == "STANDALONE",
+                     self.model_class.role == "MASTER"))).first()
 
         if not model:
             return None
 
         return model.to_data_model()
 
+    def get_vthunders_by_project_id(self, session, project_id):
+        model_list = session.query(self.model_class).filter(
+            self.model_class.project_id == project_id).filter(
+            and_(self.model_class.status == "ACTIVE",
+                 or_(self.model_class.role == "STANDALONE",
+                     self.model_class.role == "MASTER")))
+        id_list = [model.id for model in model_list]
+        return id_list
+
+    def get_vthunders_by_ip_address(self, session, ip_address):
+        model_list = session.query(self.model_class).filter(
+            self.model_class.ip_address == ip_address).filter(
+            and_(self.model_class.status == "ACTIVE",
+                 or_(self.model_class.role == "STANDALONE",
+                     self.model_class.role == "MASTER")))
+
+        id_list = [model.id for model in model_list]
+        return id_list
+
     def get_delete_compute_flag(self, session, compute_id):
         if compute_id:
             count = session.query(self.model_class).filter(
@@ -270,7 +297,6 @@ class VThunderRepository(BaseRepository):
 
         query = session.query(self.model_class).filter(
             self.model_class.updated_at < expiry_time)
-        query = session.query(self.model_class)
         if hasattr(self.model_class, 'status'):
             query = query.filter(or_(self.model_class.status == "USED_SPARE",
                                      self.model_class.status == consts.DELETED))
@@ -283,22 +309,62 @@ class VThunderRepository(BaseRepository):
         id_list = [model.id for model in model_list]
         return id_list
 
+    def get_project_list_using_partition(self, session, partition_name, ip_address):
+        queryset_vthunders = session.query(self.model_class.project_id.distinct()).filter(
+            and_(self.model_class.partition_name == partition_name,
+                 self.model_class.ip_address == ip_address,
+                 or_(self.model_class.role == "STANDALONE",
+                     self.model_class.role == "MASTER")))
+        list_projects = [project[0] for project in queryset_vthunders]
+        return list_projects
 
-class LoadBalancerRepository(BaseRepository):
-    model_class = base_models.LoadBalancer
+    def update_last_write_mem(self, session, ip_address, partition, **model_kwargs):
+        with session.begin(subtransactions=True):
+            session.query(self.model_class).filter_by(ip_address=ip_address, partition_name=partition).update(model_kwargs)
+
+
+class LoadBalancerRepository(repo.LoadBalancerRepository):
+    thunder_model_class = models.VThunder
+
+    def get_active_lbs_by_thunder(self, session, vthunder):
+        lb_list = []
+        query = session.query(self.model_class).filter(
+            and_(vthunder.loadbalancer_id == self.model_class.id,
+                 self.model_class.provisioning_status == consts.ACTIVE))
+        model_list = query.all()
+        for data in model_list:
+            lb_list.append(data.to_data_model())
+        return lb_list
+
+    def get_lb_count_by_subnet(self, session, project_ids, subnet_id):
+        return session.query(self.model_class).join(base_models.Vip).filter(
+            and_(self.model_class.project_id.in_(project_ids),
+                 base_models.Vip.subnet_id == subnet_id,
+                 or_(self.model_class.provisioning_status == consts.PENDING_DELETE,
+                     self.model_class.provisioning_status == consts.ACTIVE))).count()
+
+    def get_lb_count_by_flavor(self, session, project_id, flavor_id):
+        return session.query(self.model_class).filter(
+            self.model_class.project_id == project_id).filter(
+                self.model_class.flavor_id == flavor_id,
+                or_(self.model_class.provisioning_status == consts.PENDING_DELETE,
+                    self.model_class.provisioning_status == consts.ACTIVE)).count()
 
 
 class VRIDRepository(BaseRepository):
     model_class = models.VRID
 
-    def get_vrid_from_project_id(self, session, project_id):
+    # A project can have multiple VRIDs, so need to convert each vrid object through
+    # "to_data_model"
+    def get_vrid_from_project_ids(self, session, project_ids):
+        vrid_obj_list = []
+
         model = session.query(self.model_class).filter(
-            self.model_class.project_id == project_id).first()
+            self.model_class.project_id.in_(project_ids))
+        for data in model:
+            vrid_obj_list.append(data.to_data_model())
 
-        if not model:
-            return None
-
-        return model.to_data_model()
+        return vrid_obj_list
 
 
 class MemberRepository(repo.MemberRepository):
@@ -309,3 +375,44 @@ class MemberRepository(repo.MemberRepository):
                  or_(self.model_class.provisioning_status == consts.PENDING_DELETE,
                      self.model_class.provisioning_status == consts.ACTIVE))).count()
         return count
+
+    def get_member_count_by_subnet(self, session, project_ids, subnet_id):
+        return session.query(self.model_class).filter(
+            and_(self.model_class.project_id.in_(project_ids),
+                 self.model_class.subnet_id == subnet_id,
+                 or_(self.model_class.provisioning_status == consts.PENDING_DELETE,
+                     self.model_class.provisioning_status == consts.ACTIVE))).count()
+
+    def get_member_count_by_ip_address(self, session, ip_address, project_id):
+        return session.query(self.model_class).filter(
+            self.model_class.project_id == project_id).filter(
+            and_(self.model_class.ip_address == ip_address,
+                 or_(self.model_class.provisioning_status == consts.PENDING_DELETE,
+                     self.model_class.provisioning_status == consts.ACTIVE))).count()
+
+    def get_member_count_by_ip_address_port_protocol(self, session, ip_address, project_id, port,
+                                                     protocol):
+        return session.query(self.model_class).join(base_models.Pool).filter(
+            self.model_class.project_id == project_id).filter(
+            and_(self.model_class.ip_address == ip_address,
+                 self.model_class.protocol_port == port,
+                 base_models.Pool.protocol == protocol,
+                 or_(self.model_class.provisioning_status == consts.PENDING_DELETE,
+                     self.model_class.provisioning_status == consts.ACTIVE))).count()
+
+    def get_pool_count_by_ip(self, session, ip_address, project_id):
+        return session.query(self.model_class.pool_id.distinct()).filter(
+            self.model_class.project_id == project_id).filter(
+            and_(self.model_class.ip_address == ip_address,
+                 or_(self.model_class.provisioning_status == consts.PENDING_DELETE,
+                     self.model_class.provisioning_status == consts.ACTIVE))).count()
+
+    def get_pool_count_subnet(self, session, project_ids, subnet_id):
+        return session.query(self.model_class.pool_id.distinct()).filter(
+            self.model_class.project_id.in_(project_ids)).filter(
+            and_(self.model_class.subnet_id == subnet_id,
+                 or_(self.model_class.provisioning_status == consts.PENDING_DELETE,
+                     self.model_class.provisioning_status == consts.ACTIVE))).count()
+
+class NatPoolRepository(BaseRepository):
+    model_class = models.NATPool
