@@ -17,8 +17,10 @@ MOCK_HM_ID = 'mock-hm-1'
 MOCK_POOL_ID = 'mock-pool-1'
 MOCK_MEMBER_ID = 'mock-member-1'
 MOCK_LOAD_BALANCER_ID = 'mock-lb-1'
-MOCK_COOKIE_NAME = 'mock-cookie-1'
 MOCK_LISTENER_ID = 'mock-listener-1'
+MOCK_FLAVOR_ID = 'mock-flavor-1'
+MOCK_FLAVOR_PROF_ID = 'mock-flavor-prof-1'
+MOCK_COOKIE_NAME = 'mock-cookie-1'
 MOCK_CERT_FILENAME = 'mock-cert-filename-1'
 MOCK_CERT_CONTENT = 'mock-cert-content-1'
 MOCK_KEY_FILENAME = 'mock-key-filename-1'
@@ -35,6 +37,7 @@ MOCK_VRID_FULL_FLOATING_IP = '10.10.10.45'
 MOCK_SUBNET_CIDR = '10.10.10.0/24'
 MOCK_SUBNET_ID = 'mock-subnet-1'
 MOCK_VRRP_PORT_ID = 'mock-vrrp-port-id'
+MOCK_PORT_ID = 'mock-port-id'
 
 SERVICE_GROUP_CONF_SECTION = 'service_group'
 LISTENER_CONF_SECTION = 'listener'
@@ -51,7 +54,16 @@ A10_HARDWARE_THUNDER_CONF_SECTION = 'hardware_thunder'
 LISTENER_CONF_SECTION = 'listener'
 SERVICE_GROUP_CONF_SECTION = 'service_group'
 SERVER_CONF_SECTION = 'server'
+VIRTUAL_SERVER_CONFIG_SECTION = 'slb'
+HEALTH_MONITOR_SECTION = 'health_monitor'
+
+A10_HOUSE_KEEPING = 'a10_house_keeping'
 
 MOCK_SERVICE_GROUP_PROTOCOL = "HTTP"
 MOCK_POOL_ID_2 = "mock-pool-2"
 MOCK_SUBNET_ID_2 = "mock-subnet-2"
+
+MOCK_L7POLICY_ID = "mock-l7-policy-1"
+MOCK_L7RULE_ID = "mock-l7-rule-1"
+
+MOCK_TLS_CERTIFICATE_ID = "mock_tls_certificate_id"
