@@ -45,22 +45,32 @@ HW_THUNDER = data_models.HardwareThunder(
     password="abc",
     ip_address="10.10.10.10",
     partition_name="shared")
-LB = o_data_models.LoadBalancer(id=a10constants.MOCK_LOAD_BALANCER_ID)
+LB = o_data_models.LoadBalancer(id=a10constants.MOCK_LOAD_BALANCER_ID,
+                                flavor_id=a10constants.MOCK_FLAVOR_ID)
 FIXED_IP = n_data_models.FixedIP(ip_address='10.10.10.10')
 PORT = n_data_models.Port(id=uuidutils.generate_uuid(), fixed_ips=[FIXED_IP])
 VRID = data_models.VRID(id=1, vrid=0,
                         project_id=a10constants.MOCK_PROJECT_ID,
                         vrid_port_id=uuidutils.generate_uuid(),
                         vrid_floating_ip='10.0.12.32')
+LISTENER = o_data_models.Listener(id=a10constants.MOCK_LISTENER_ID, load_balancer=LB)
+POOL = o_data_models.Pool(id=a10constants.MOCK_POOL_ID, load_balancer=LB)
+HM = o_data_models.HealthMonitor(id=a10constants, pool=POOL)
 MEMBER_1 = o_data_models.Member(id=uuidutils.generate_uuid(),
                                 project_id=a10constants.MOCK_PROJECT_ID,
-                                subnet_id=a10constants.MOCK_SUBNET_ID)
+                                subnet_id=a10constants.MOCK_SUBNET_ID,
+                                pool=POOL)
 MEMBER_2 = o_data_models.Member(id=uuidutils.generate_uuid(),
                                 project_id=a10constants.MOCK_PROJECT_ID,
-                                subnet_id=a10constants.MOCK_SUBNET_ID_2)
-
-POOL = o_data_models.Pool(id=a10constants.MOCK_POOL_ID)
+                                subnet_id=a10constants.MOCK_SUBNET_ID_2,
+                                pool=POOL)
+FLAVOR_PROFILE = o_data_models.FlavorProfile(id=a10constants.MOCK_FLAVOR_PROF_ID,
+                                             flavor_data={})
+FLAVOR = o_data_models.Flavor(id=a10constants.MOCK_FLAVOR_ID,
+                              flavor_profile_id=a10constants.MOCK_FLAVOR_PROF_ID)
 SUBNET = n_data_models.Subnet(id=uuidutils.generate_uuid())
+NAT_POOL = data_models.NATPool(id=uuidutils.generate_uuid(),
+                               port_id=a10constants.MOCK_PORT_ID)
 
 
 class TestA10DatabaseTasks(base.BaseTaskTestCase):
@@ -69,6 +79,7 @@ class TestA10DatabaseTasks(base.BaseTaskTestCase):
         super(TestA10DatabaseTasks, self).setUp()
         imp.reload(task)
         self.vrid_repo = mock.Mock()
+        self.nat_pool_repo = mock.Mock()
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         self.conf.register_opts(config_options.A10_GLOBAL_OPTS,
                                 group=a10constants.A10_GLOBAL_CONF_SECTION)
@@ -362,8 +373,239 @@ class TestA10DatabaseTasks(base.BaseTaskTestCase):
     def test_get_child_projects_for_partition(self):
         vthunder = copy.deepcopy(VTHUNDER)
         vthunder.partition_name = "mock-partition-name"
+        vthunder.ip_address = "mock-ip-addr"
         mock_get_projects = task.GetChildProjectsOfParentPartition()
         mock_get_projects.vthunder_repo.get_project_list_using_partition = mock.Mock()
         mock_get_projects.execute(MEMBER_1, vthunder)
         mock_get_projects.vthunder_repo.get_project_list_using_partition.\
-            assert_called_once_with(mock.ANY, partition_name='mock-partition-name')
+            assert_called_once_with(mock.ANY, partition_name='mock-partition-name',
+                                    ip_address="mock-ip-addr")
+
+    def test_flavor_search_loadbalancer_find_flavor(self):
+        flavor_task = task.GetFlavorData()
+        found_id = flavor_task._flavor_search(LB)
+        self.assertEqual(found_id, a10constants.MOCK_FLAVOR_ID)
+
+    def test_flavor_search_listener_find_flavor(self):
+        flavor_task = task.GetFlavorData()
+        found_id = flavor_task._flavor_search(LB)
+        self.assertEqual(found_id, a10constants.MOCK_FLAVOR_ID)
+
+    def test_flavor_search_pool_find_flavor(self):
+        flavor_task = task.GetFlavorData()
+        found_id = flavor_task._flavor_search(POOL)
+        self.assertEqual(found_id, a10constants.MOCK_FLAVOR_ID)
+
+    def test_flavor_search_member_find_flavor(self):
+        flavor_task = task.GetFlavorData()
+        found_id = flavor_task._flavor_search(MEMBER_1)
+        self.assertEqual(found_id, a10constants.MOCK_FLAVOR_ID)
+
+    def test_flavor_search_health_monitor_find_flavor(self):
+        flavor_task = task.GetFlavorData()
+        found_id = flavor_task._flavor_search(HM)
+        self.assertEqual(found_id, a10constants.MOCK_FLAVOR_ID)
+
+    def test_GetFlavorData_format_flavor_keys(self):
+        expected = {"virtual_server": {"arp_disable": 1}}
+        flavor = {"virtual-server": {"arp-disable": 1}}
+        flavor_task = task.GetFlavorData()
+        formated_flavor = flavor_task._format_keys(flavor)
+        self.assertEqual(formated_flavor, expected)
+
+    def test_GetFlavorData_format_flavor_list_keys(self):
+        expected = {
+            "service_group": {
+                "name_expressions": [{
+                    "regex": "sg1",
+                    "json": {"health_check_disable": 1}
+                }]
+            }
+        }
+        name_expr = {
+            "name-expressions": [{
+                "regex": "sg1",
+                "json": {"health-check-disable": 1}
+            }]
+        }
+        flavor = {"service-group": name_expr}
+        flavor_task = task.GetFlavorData()
+        formated_flavor = flavor_task._format_keys(flavor)
+        self.assertEqual(formated_flavor, expected)
+
+    def test_GetFlavorData_format_flavor_multi_val(self):
+        expected = {
+            "service_group": {
+                "name_expressions": [{
+                    "regex": "sg1",
+                    "json": {"health_check_disable": 1}
+                }],
+                "strict_select": 0
+            }
+        }
+        name_expr = {
+            "name-expressions": [{
+                "regex": "sg1",
+                "json": {"health-check-disable": 1}
+            }]
+        }
+        flavor = {"service-group": {"strict-select": 0}}
+        flavor['service-group'].update(name_expr)
+        flavor_task = task.GetFlavorData()
+        formated_flavor = flavor_task._format_keys(flavor)
+        self.assertEqual(formated_flavor, expected)
+
+    def test_GetFlavorData_execute_no_flavor_id(self):
+        flavor_task = task.GetFlavorData()
+        flavor_task._flavor_search = mock.Mock(return_value=None)
+        ret_val = flavor_task.execute(LB)
+        self.assertEqual(ret_val, None)
+
+    def test_GetFlavorData_execute_no_flavor_or_profile(self):
+        flavor_task = task.GetFlavorData()
+        flavor_task.flavor_repo = mock.Mock()
+        flavor_task.flavor_repo.get.return_value = None
+        flavor_task._flavor_search = mock.Mock(
+            return_value=a10constants.MOCK_FLAVOR_ID)
+        ret_val = flavor_task.execute(LB)
+        self.assertEqual(ret_val, None)
+
+        flavor_task.flavor_repo.reset_mock()
+        flavor_task.flavor_repo = mock.Mock()
+        flavor = copy.deepcopy(FLAVOR)
+        flavor.flavor_profile_id = None
+        flavor_task.flavor_repo.get.return_value = flavor
+        ret_val = flavor_task.execute(LB)
+        self.assertEqual(ret_val, None)
+
+    def test_GetFlavorData_execute_return_flavor(self):
+        flavor_task = task.GetFlavorData()
+        flavor_task._flavor_search = mock.Mock(
+            return_value=a10constants.MOCK_FLAVOR_ID)
+        flavor_task._format_keys = mock.Mock()
+        flavor_task._format_keys.return_value = {}
+        flavor_task.flavor_repo = mock.Mock()
+        flavor_task.flavor_repo.get.return_value = FLAVOR
+
+        flavor_prof = copy.deepcopy(FLAVOR_PROFILE)
+        flavor_prof.flavor_data = "{}"
+        flavor_task.flavor_profile_repo = mock.Mock()
+        flavor_task.flavor_profile_repo.get.return_value = flavor_prof
+        ret_val = flavor_task.execute(LB)
+        self.assertEqual(ret_val, {})
+
+    def test_GetNatPoolEntry(self):
+        db_task = task.GetNatPoolEntry()
+        db_task.nat_pool_repo = self.nat_pool_repo
+        flavor = {"pool_name": "p1", "start_address": "1.1.1.1", "end_address": "1.1.1.2"}
+        db_task.execute(MEMBER_1, flavor)
+        self.nat_pool_repo.get(mock.ANY, name="p1", subnet_id=MEMBER_1.subnet_id)
+
+    def test_UpdateNatPoolDB_update(self):
+        db_task = task.UpdateNatPoolDB()
+        db_task.nat_pool_repo = self.nat_pool_repo
+        flavor = {"pool_name": "p1", "start_address": "1.1.1.1", "end_address": "1.1.1.2"}
+        NAT_POOL.member_ref_count = 1
+        db_task.execute(MEMBER_1, flavor, NAT_POOL, None)
+        self.nat_pool_repo.update.assert_called_with(mock.ANY, mock.ANY, member_ref_count=2)
+
+    def test_UpdateNatPoolDB_create(self):
+        db_task = task.UpdateNatPoolDB()
+        db_task.nat_pool_repo = self.nat_pool_repo
+        flavor = {"pool_name": "p1", "start_address": "1.1.1.1", "end_address": "1.1.1.2"}
+        port = mock.Mock(id="1")
+        db_task.execute(MEMBER_1, flavor, None, port)
+        self.nat_pool_repo.create.assert_called_with(
+            mock.ANY, id=mock.ANY, name="p1",
+            subnet_id=MEMBER_1.subnet_id, start_address="1.1.1.1", end_address="1.1.1.2",
+            member_ref_count=1, port_id="1")
+
+    def test_DeleteNatPoolEntry_update(self):
+        db_task = task.DeleteNatPoolEntry()
+        db_task.nat_pool_repo = self.nat_pool_repo
+        NAT_POOL.member_ref_count = 2
+        db_task.execute(NAT_POOL)
+        self.nat_pool_repo.update.assert_called_with(mock.ANY, mock.ANY, member_ref_count=1)
+
+    def test_DeleteNatPoolEntry_delete(self):
+        db_task = task.DeleteNatPoolEntry()
+        db_task.nat_pool_repo = self.nat_pool_repo
+        NAT_POOL.member_ref_count = 1
+        db_task.execute(NAT_POOL)
+        self.nat_pool_repo.delete(mock.ANY, id=NAT_POOL.id)
+
+    def test_lb_count_according_to_flavor(self):
+        mock_count_lb = task.CountLoadbalancersWithFlavor()
+        mock_count_lb.loadbalancer_repo.get_lb_count_by_flavor = mock.Mock()
+        mock_count_lb.loadbalancer_repo.get_lb_count_by_flavor.return_value = 2
+        lb_count = mock_count_lb.execute(LB)
+        self.assertEqual(2, lb_count)
+
+    def test_SetThunderUpdatedAt_execute_update(self):
+        db_task = task.SetThunderUpdatedAt()
+        vthunder = copy.deepcopy(VTHUNDER)
+        db_task.vthunder_repo.update = mock.Mock()
+        db_task.execute(vthunder)
+        db_task.vthunder_repo.update.assert_called_once_with(mock.ANY,
+                                                             vthunder.id,
+                                                             updated_at=mock.ANY)
+
+    def test_SetThunderLastWriteMem_execute_update(self):
+        db_task = task.SetThunderLastWriteMem()
+        vthunder = copy.deepcopy(VTHUNDER)
+        db_task.vthunder_repo.update_last_write_mem = mock.Mock()
+        db_task.execute(vthunder, True, True)
+        db_task.vthunder_repo.update_last_write_mem.assert_called_once_with(
+            mock.ANY, vthunder.ip_address, vthunder.partition_name, last_write_mem=mock.ANY)
+
+    def test_GetActiveLoadBalancersByThunder_return_empty(self):
+        lb_task = task.GetActiveLoadBalancersByThunder()
+        vthunder = copy.deepcopy(VTHUNDER)
+        vthunder.loadbalancer_id = a10constants.MOCK_LOAD_BALANCER_ID
+        lb_task.loadbalancer_repo.get_active_lbs_by_thunder = mock.Mock()
+        lb_task.loadbalancer_repo.get_active_lbs_by_thunder.return_value = []
+        lb_list = lb_task.execute(vthunder)
+        self.assertEqual(lb_list, [])
+
+    def test_GetActiveLoadBalancersByThunder_return_list(self):
+        lb_task = task.GetActiveLoadBalancersByThunder()
+        vthunder = copy.deepcopy(VTHUNDER)
+        vthunder.loadbalancer_id = a10constants.MOCK_LOAD_BALANCER_ID
+        lb_task.loadbalancer_repo.get_active_lbs_by_thunder = mock.Mock()
+        lb_task.loadbalancer_repo.get_active_lbs_by_thunder.return_value = [LB]
+        lb_list = lb_task.execute(vthunder)
+        self.assertEqual(len(lb_list), 1)
+
+    def test_MarkLoadBalancersPendingUpdateInDB_execute(self):
+        lb_task = task.MarkLoadBalancersPendingUpdateInDB()
+        lb_list = []
+        lb_list.append(LB)
+        lb_task.loadbalancer_repo.update = mock.Mock()
+        lb_task.execute(lb_list)
+        lb_task.loadbalancer_repo.update.assert_called_once_with(mock.ANY,
+                                                                 LB.id,
+                                                                 provisioning_status=mock.ANY)
+
+    def test_MarkLoadBalancersPendingUpdateInDB_execute_for_empty_list(self):
+        lb_task = task.MarkLoadBalancersPendingUpdateInDB()
+        lb_list = []
+        lb_task.loadbalancer_repo.update = mock.Mock()
+        lb_task.execute(lb_list)
+        lb_task.loadbalancer_repo.update.assert_not_called()
+
+    def test_MarkLoadBalancersActiveInDB_execute(self):
+        lb_task = task.MarkLoadBalancersActiveInDB()
+        lb_list = []
+        lb_list.append(LB)
+        lb_task.loadbalancer_repo.update = mock.Mock()
+        lb_task.execute(lb_list)
+        lb_task.loadbalancer_repo.update.assert_called_once_with(mock.ANY,
+                                                                 LB.id,
+                                                                 provisioning_status=mock.ANY)
+
+    def test_MarkLoadBalancersActiveInDB_execute_for_empty_list(self):
+        lb_task = task.MarkLoadBalancersActiveInDB()
+        lb_list = []
+        lb_task.loadbalancer_repo.update = mock.Mock()
+        lb_task.execute(lb_list)
+        lb_task.loadbalancer_repo.update.assert_not_called()
